CON
  _clkmode = xtal1 + pll16x     
  _xinfreq = 5_000_000

  BUFFER_2K     = $800
  
  CR            = $0D
  LF            = $0A

  SNTP_PORT     = 123
  SNTP_SOCK     = 0
  

      'USA Standard Time Zone Abbreviations
 #-10, HST,AtST,_PST,MST,CST,EST,AlST
              
    'USA Daylight Time Zone Abbreviations
  #-9, HDT,AtDT,PDT,MDT,CDT,EDT,AlDT

  Zone = MST 

       
VAR
  long  longHIGH, longLOW, MM_DD_YYYY, DW_HH_MM_SS 'Expected 4-contigous variables

DAT
  buff          byte  $0[BUFFER_2K]
  null          byte  $0

  
OBJ
  pst           : "Parallax Serial Terminal"
  sock          : "Socket"
  wiz           : "W5100"
  sntp          : "SNTP Simple Network Time Protocol v2.01" 


 
PUB Main | bytesToRead, bytesSent, receiving, ptr

  receiving := true
  bytesToRead := 0
  pst.Start(115_200)
  pause(500)

  'W5100 I/O
  wiz.Start(WIZ#SPI_CS, WIZ#SPI_SCK, WIZ#SPI_MOSI, WIZ#SPI_MISO)
  
  'Set network parameters
  wiz.SetCommonnMode(0)
  wiz.SetGateway(192, 168, 1, 1)
  wiz.SetSubnetMask(255, 255, 255, 0)
  wiz.SetIp(192, 168, 1, 109)
  wiz.SetMac($00, $08, $DC, $16, $F1, $32)
  
  
  pst.str(string("Initialize", CR))
  sock.Init(SNTP_SOCK, wiz#UDP, SNTP_PORT)


  sock.RemoteIp(64, 147, 116, 229)
  sock.RemotePort(SNTP_PORT)

  'Create header
  sntp.CreateUDPtimeheader(@buff)
  DisplayMemory(@buff, 48, true)

  pst.str(string("Send Message",CR))

  ptr := SendReceive(@buff, 48)
  if(ptr ==  @null)
    pst.str(string(CR, "Timeout", CR))
  else
    DisplayUdpHeader(@buff)
    DisplayMemory(ptr, 48, true)
    SNTP.GetTransmitTimestamp(Zone,@buff,@LongHIGH,@LongLOW)
    DisplayHumanTime

    
PUB SendReceive(buffer, len) | bytesToRead, ptr 
  
  bytesToRead := 0

  'Open socket and Send Message
  pst.str(string("Open/Send",CR))
  sock.Open
  sock.Send(buffer, len)

  pause(500)
  
  bytesToRead := sock.Available
   
  'Check for a timeout
  if(bytesToRead =< 0 )
    bytesToRead~
    return @null

  if(bytesToRead > 0) 
    'Get the Rx buffer  
    ptr := sock.Receive(buffer, bytesToRead)

  sock.Disconnect
  return ptr

PUB DisplayHumanTime
    pst.Char(13)
    pst.Char(13)
    pst.Char(13)
    pst.str(string("SNTP Server Sync time:",13))
    pst.str(string("(GMT "))
    if Zone<0
       pst.Char("-")
    else
       pst.Char("+")
    pst.str(string(" ",||Zone+48,":00) "))
    if byte[@MM_DD_YYYY][3]<10
       pst.Char("0")
    pst.dec(byte[@MM_DD_YYYY][3])
    pst.Char("/")
    if byte[@MM_DD_YYYY][2]<10
       pst.Char("0")
    pst.dec(byte[@MM_DD_YYYY][2])
    pst.Char("/")
    pst.dec(word[@MM_DD_YYYY][0])                    
    pst.Char($20)
    if byte[@DW_HH_MM_SS][2]<10
       pst.Char("0")
    pst.dec(byte[@DW_HH_MM_SS][2])
    pst.Char(":")
    if byte[@DW_HH_MM_SS][1]<10
       pst.Char("0")
    pst.dec(byte[@DW_HH_MM_SS][1])
    pst.Char(":")
    if byte[@DW_HH_MM_SS][0]<10
       pst.Char("0")
    pst.dec(byte[@DW_HH_MM_SS][0])
    pst.Char(13)
      
PUB DisplayUdpHeader(buffer)
  pst.char(CR)
  pst.str(string(CR, "Message from......."))
  PrintIp(buffer)
  pst.char(":")
  pst.dec(DeserializeWord(buffer + 4))
  pst.str(string(" ("))
  pst.dec(DeserializeWord(buffer + 6))
  pst.str(string(")", CR))
      
PUB DisplayMemory(addr, len, isHex) | j
  pst.str(string(13,"-----------------------------------------------------",13))
  pst.str(string(13, "      "))
  repeat j from 0 to $F
    pst.hex(j, 2)
    pst.char($20)
  pst.str(string(13, "      ")) 
  repeat j from 0 to $F
    pst.str(string("-- "))

  pst.char(13) 
  repeat j from 0 to len-1
    if(j == 0)
      pst.hex(0, 4)
      pst.char($20)
      pst.char($20)
      
    if(isHex)
      pst.hex(byte[addr + j], 2)
    else
      pst.char($20)
      if(byte[addr+j] == 0)
        pst.char($20)
      pst.char(byte[addr+j])

    pst.char($20) 
    if((j+1) // $10 == 0) 
      pst.char($0D)
      pst.hex(j+1, 4)
      pst.char($20)
      pst.char($20)  
  pst.char(13)
  
  pst.char(13)
  pst.str(string("Start: "))
  pst.dec(addr)
  pst.str(string(" Len: "))
  pst.dec(len)
  pst.str(string(13,"-----------------------------------------------------",13,13))
      
PUB PrintIp(addr) | i
  repeat i from 0 to 3
    pst.dec(byte[addr][i])
    if(i < 3)
      pst.char($2E)
    'else
      'pst.char($0D)
      
      
PRI SerializeWord(value, buffer)
  byte[buffer++] := (value & $FF00) >> 8
  byte[buffer] := value & $FF

PRI DeserializeWord(buffer) | value
  value := byte[buffer++] << 8
  value += byte[buffer]
  return value
        
PRI pause(Duration)  
  waitcnt(((clkfreq / 1_000 * Duration - 3932) #> 381) + cnt)
  return

  {{

    receiving := true
  repeat while receiving 
    'Data in the buffer?
    bytesToRead := sock.Available
    pst.str(string("Bytes to Read: "))
    pst.dec(bytesToRead)
    pst.char(13)
    pst.char(13)
     
    'Check for a timeout
    if(bytesToRead == -1)
      receiving := false
      pst.str(string("Fail safe", CR))
      next

    if(bytesToRead == 0)
      receiving := false
      pst.str(string("Done", CR))
      next 

    if(bytesToRead > 0) 
      'Get the Rx buffer  
      ptr := sock.Receive(@buff, bytesToRead)
      pst.char(CR)
      pst.str(string("UPD Header:",CR))
      PrintIp(@buff)
      pst.dec(DeserializeWord(@buff + 4))
      pst.char(CR)
      pst.dec(DeserializeWord(@buff + 6))
      pst.char(CR)
       
      pst.char(CR) 
      DisplayMemory(ptr, DeserializeWord(@buff + 6), true) 
      pst.char(CR)
      
    bytesToRead~

  pst.str(string(CR, "Disconnect", CR)) 
  sock.Disconnect

  }}